from twisted.internet import reactor, protocol 
import os

from twisted.internet.protocol import connectionDone
from twisted.python import failure

class FileTransferProtocol(protocol.Protocol): 
    def connectionMade(self): 
        print("Client connected.") 

    def dataReceived(self, data): 
        if data == b"SEND": 
            self.transport.write(b"READY") 
            self.transferFile = True 
        elif self.transferFile: 
            with open("received_file.txt", "wb") as f: 
                f.write(data) 
            self.transport.write(b"RECEIVED") 
            self.transferFile = False
        else: 
            self.transport.write(b"ERROR")
    
    def connectionLost(self, reason):
        print("Client Disconnected")
            

class FileTransferFactory(protocol.Factory): 
    protocol = FileTransferProtocol 

    
if __name__ == "__main__": 
    reactor.listenTCP(7000, FileTransferFactory()) 
    print("Server started.")
    reactor.run() 



from twisted.internet import reactor
from twisted.web.client import Agent
from twisted.web.client import readBody


def download_web_page(url):
    agent = Agent(reactor)

    def handle_response(response):
        d = readBody(response)
        d.addCallback(lambda body: print(body.decode()))
        d.addBoth(lambda _: reactor.stop())

    def handle_error(error):
        print(f"An error occurred: {error}")
        reactor.stop()

    d = agent.request(b"GET", url.encode())
    d.addCallbacks(handle_response, handle_error)

    reactor.run()


if __name__ == "__main__":
    download_web_page("http://www.google.com/")




from twisted.internet import reactor, protocol
from twisted.internet.protocol import connectionDone
from twisted.python import failure 
class EchoClient(protocol.Protocol): 
    def connectionMade(self):
        msg = input("Enter the message to Server - ") 
        self.transport.write(msg.encode()) 

    def dataReceived(self, data):
        print ("Acknoledgement from Server -", data.decode()) 
        self.transport.loseConnection()

class EchoFactory(protocol.ClientFactory): 
    def buildProtocol(self, addr): 
        return EchoClient()
        
reactor.connectTCP("localhost", 8000, EchoFactory())
reactor.run() 



from twisted.internet import reactor, protocol 
import os

class FileTransferClientProtocol(protocol.Protocol):
    def connectionMade(self): 
        try: 
            f = open("myfile.txt", "rb") 
            self.fileData = f.read() 
            f.close()
            if len(self.fileData) == 0: 
                print("File is empty.") 
                self.transport.loseConnection() 
            else: 
                self.transport.write(b"SEND") 
        except FileNotFoundError: 
            print("File not found.") 
            self.transport.loseConnection()

    def dataReceived(self, data): 
        if data == b"READY": 
            if self.fileData: 
                self.transport.write(self.fileData) 
            else: 
                self.transport.loseConnection() 
        elif data == b"RECEIVED":
            print("File transfer complete.") 
            self.transport.loseConnection() 
        else:
            print("Error:", data.decode()) 
            self.transport.loseConnection() 

class FileTransferClientFactory(protocol.ClientFactory): 
    protocol = FileTransferClientProtocol 

if __name__ == "__main__":
    reactor.connectTCP("localhost", 7000, FileTransferClientFactory()) 
    reactor.run() 



from twisted.internet import reactor, protocol
from twisted.protocols.basic import LineOnlyReceiver 

class ChatProtocol(LineOnlyReceiver): 
    def __init__(self, factory): 
        self.factory = factory 
        self.name = None 
        self.state = "GETNAME" 

    def lineReceived(self, line):
        if self.state == "GETNAME": 
            self.handle_GETNAME(line.decode()) 
        else: 
            self.handle_CHAT(line.decode()) 

    def handle_GETNAME(self, name): 
        if name in self.factory.users: 
            self.sendLine("Name already taken, please choose another name.".encode()) 
            return 
        self.sendLine(f"Welcome, {name}!".encode()) 
        self.broadcastMessage(f"{name} has joined the chat room.") 
        self.name = name
        self.factory.users[name] = self
        self.state = "CHAT" 

    def handle_CHAT(self, message): 
        if message.lower() == "/quit": 
            self.transport.loseConnection() 
        else: 
            message = f"<{self.name}> {message}" 
            self.broadcastMessage(message) 

    def broadcastMessage(self, message):
        for name, protocol in self.factory.users.items(): 
            if protocol != self: 
                protocol.sendLine(message.encode()) 

    def connectionMade(self): 
        self.sendLine("Connected to the chat server. Type '/quit' to exit.".encode()) 
        self.factory.clients.append(self) 


    def connectionLost(self, reason): 
        self.factory.clients.remove(self)
        print("Client Disconnected")

class ChatFactory(protocol.Factory): 
    def __init__(self): 
        self.users = {} 
        self.clients = [] 

    def buildProtocol(self, addr): 
        return ChatProtocol(self) 
        
            
if __name__ == "__main__": 
    reactor.listenTCP(9000, ChatFactory()) 
    print("Chat server started. Listening on port 9000...") 
    reactor.run() 


from twisted.internet import reactor, protocol
from twisted.protocols.basic import LineOnlyReceiver
import ipaddress

SUBNET = ipaddress.IPv4Network("192.168.0.0/24")

class SubnetCheckerProtocol(LineOnlyReceiver):
    def connectionMade(self):
        self.sendLine(b"Enter an IP address to check:")

    def lineReceived(self, line):
        ip_address = line.strip().decode()
        result = b"IP address is within the subnet" if self.is_in_subnet(ip_address) else b"IP address is outside the subnet"
        self.sendLine(result)
        self.transport.loseConnection()

    def is_in_subnet(self, ip_address):
        try:
            return ipaddress.IPv4Address(ip_address) in SUBNET
        except ipaddress.AddressValueError:
            return False

if __name__ == "__main__":
    reactor.listenTCP(8000, protocol.Factory.forProtocol(SubnetCheckerProtocol))
    print("Subnet checker server is running...")
    reactor.run()



from twisted.internet import protocol, reactor


class SelectiveRepeatProtocol(protocol.Protocol):
    def __init__(self):
        self.window_size = 4  # Size of the sender and receiver window
        self.send_base = 0  # Sequence number of the oldest unacknowledged packet
        self.send_next = 0  # Sequence number of the next packet to be sent
        self.recv_base = 0  # Sequence number of the oldest unprocessed packet

        self.buffer = ['Packet0', 'Packet1', 'Packet2', 'Packet3'] 

    def connectionMade(self):
        print("Connection established.")
        self.sendPackets()

    def dataReceived(self, data):
        # Simulate packet loss
        if data != b"ACK":
            print("Packet lost!")
            return

        print("ACK received.")
        self.send_base += 1
        self.send_next += 1
        self.sendPackets()

    def sendPackets(self):
        while self.send_next < self.send_base + self.window_size and self.send_next < len(self.buffer):
            packet = self.buffer[self.send_next]
            print("Sending packet:", packet)
            self.transport.write(packet.encode())
            self.send_next += 1

    def connectionLost(self, reason):
        print("Connection lost.")


class SelectiveRepeatFactory(protocol.Factory):
    def buildProtocol(self, addr):
        return SelectiveRepeatProtocol()

reactor.listenTCP(8000, SelectiveRepeatFactory())
reactor.run()



from twisted.internet import reactor, protocol

class StopAndWaitServer(protocol.Protocol):
    def connectionMade(self):
        print("Client connected:", self.transport.getPeer())
        self.send_message()

    def send_message(self):
        message = input("Enter message: ")
        self.transport.write(message.encode())
        print("Message sent to client:", message)
        self.expected_ack = "ACK"
        # self.schedule_resend()

    def schedule_resend(self):
        self.resend_call = reactor.callLater(5, self.resend_message)

    def resend_message(self):
        print("ACK not received. Resending message...")
        self.send_message()

    def dataReceived(self, data):
        ack = data.decode()
        print("ACK received:", ack)
        if ack == self.expected_ack:
            # self.resend_call.cancel()
            print("ACK received. Message acknowledged.")
            self.send_message()
        else:
            print("Invalid ACK received.")
            self.schedule_resend()

    def connectionLost(self, reason):
        print("Client disconnected:")

class StopAndWaitServerFactory(protocol.Factory):
    def buildProtocol(self, addr):
        return StopAndWaitServer()

server_port = 8000
factory = StopAndWaitServerFactory()
reactor.listenTCP(server_port, factory)
reactor.run()



from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor

class ChatClient(DatagramProtocol):
    def startProtocol(self):
        host = "127.0.0.1"  # Change this to the IP address of your server
        port = 5000  # Make sure this matches the port your server is listening on
        username = input("Enter your username: ")

        # Send the username to the server
        self.transport.write(username.encode('utf-8'), (host, port))

        # Start sending messages
        reactor.callInThread(self.send_messages)

    def send_messages(self):
        while True:
            message = input("Enter your message (or type 'exit' to quit): ")
            if message.lower() == 'exit':
                reactor.stop()
                break
            self.transport.write(message.encode('utf-8'), ("127.0.0.1", 5000))

    def datagramReceived(self, data, addr):
        print(data.decode('utf-8'))

if __name__ == "__main__":
    reactor.listenUDP(0, ChatClient())
    reactor.run()



from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor

class FileReceiver(DatagramProtocol):
    def datagramReceived(self, datagram, address):
        with open("received_file.txt", "wb") as f:
            f.write(datagram)

if __name__ == '__main__':
    reactor.listenUDP(8000, FileReceiver())
    reactor.run()



from twisted.internet import reactor,protocol

class UDPServer(protocol.DatagramProtocol):
    def datagramReceived(self, data, addr):
        print("Recieved message from",addr[0],": ",data.decode())
        self.transport.write(data,addr)

server_port=8000
server=reactor.listenUDP(server_port,UDPServer())
print("Server started on port",server_port)

reactor.run()


from twisted.internet import protocol,reactor
table = {}
table["192.168.1.1"] = "00:11:22:33:44:55"

class dns_server(protocol.Protocol):
    def connectionMade(self):
        print('connection made')
    
    def dataReceived(self, data: bytes):
        global table
        a = data.decode()
        ip = table.get(a)
        if ip is not None:
            response = f"MAC FOR {a} is {ip}"
        else:
            response = f"NO IP FOR DOMAIN {a}"
        self.transport.write(response.encode())

class dns_factory(protocol.Factory):
    def buildProtocol(self, addr):
        return dns_server()

reactor.listenTCP(8000, dns_factory())
reactor.run()



from twisted.internet import protocol, reactor

class RARPClient(protocol.Protocol):
    def connectionMade(self):
        print('Connection made')
        self.sendNextMAC()

    def sendNextMAC(self):
        mac_address = input('Enter MAC address (or type "exit" to quit): ')
        self.transport.write(mac_address.encode())

    def dataReceived(self, data):
        response = data.decode().strip()
        print(response)
        if response.lower().startswith("exiting"):
            self.transport.loseConnection()
        else:
            self.sendNextMAC()

class RARPClientFactory(protocol.ClientFactory):
    def buildProtocol(self, addr):
        return RARPClient()

    def clientConnectionFailed(self, connector, reason):
        print(f'Connection failed: {reason.getErrorMessage()}')
        reactor.stop()

reactor.connectTCP('localhost', 8001, RARPClientFactory())
reactor.run()




from twisted.internet import protocol,reactor
from twisted.internet.protocol import connectionDone
from twisted.python import failure 

class echo(protocol.Protocol): 
    def dataReceived(self, data):
        print("Message from Client -", data.decode()) 
        print("Client Connected!")
        ack_msg = f"{data.decode()}"
        ack = "ACK[" + ack_msg + "]" 
        print("Acknoledgement Sent!") 
        self.transport.write(ack.encode()) 

    def connectionLost(self, reason):
        print("Client Disconnected")
        
class echofactory(protocol.Factory): 
    def buildProtocol(self, addr): 
        return echo() 
    
reactor.listenTCP(8000,echofactory())
reactor.run() 




from twisted.internet import reactor,protocol

class UDPClient(protocol.DatagramProtocol):
    def startProtocol(self):
        self.transport.connect('127.0.0.2',8000)
        self.send_message()
    
    def send_message(self):
        message=input('Enter message: ')
        self.transport.write(message.encode())

    def datagramReceived(self, data, addr):
        print("Recieved message: ",data.decode())
        self.send_message()
    
client=reactor.listenUDP(0,UDPClient())
reactor.run()


from twisted.internet import reactor, protocol

class RPCServerProtocol(protocol.Protocol):
    def dataReceived(self, data):
        request = data.decode().strip()
        result = self.processRequest(request)
        self.transport.write(result.encode())

    def processRequest(self, request):
        if request == "add 10 5":
            return str(10 + 5)
        else:
            return "Invalid request"

class RPCServerFactory(protocol.Factory):
    def buildProtocol(self, addr):
        return RPCServerProtocol()

if __name__ == "__main__":
    reactor.listenTCP(8000, RPCServerFactory())
    print("RPC server is running...")
    reactor.run()




from twisted.internet import reactor, protocol

class StopAndWaitClient(protocol.Protocol):
    def connectionMade(self):
        print("Connected to server.")
        self.send_ack()

    def send_ack(self):
        self.transport.write(input("Enter ack: ").encode())
        print("ACK sent")

    def dataReceived(self, data):
        message = data.decode()
        print("Message received:", message)
        self.send_ack()

    def connectionLost(self, reason):
        print("Connection lost:", reason.getErrorMessage())

class StopAndWaitClientFactory(protocol.ClientFactory):
    def buildProtocol(self, addr):
        return StopAndWaitClient()

    def clientConnectionFailed(self, connector, reason):
        print("Connection failed:", reason.getErrorMessage())
        reactor.stop()

    def clientConnectionLost(self, connector, reason):
        print("Connection lost:", reason.getErrorMessage())
        reactor.stop()

server_address = 'localhost'
server_port = 8000
factory = StopAndWaitClientFactory()
reactor.connectTCP(server_address, server_port, factory)
reactor.run()



from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor

class ChatServer(DatagramProtocol):
    def __init__(self):
        self.users = {}  # maps user addresses to usernames

    def datagramReceived(self, data, addr):
        message = data.decode('utf-8').strip()

        # Check if the message starts with a known command
        if message.startswith("/username "):
            # Extract the username from the command
            username = message.split()[1]
            self.users[addr] = username
            self.transport.write(f"Username set to {username}\n".encode('utf-8'), addr)
        else:
            # Normal chat message
            username = self.users.get(addr, "Unknown")
            broadcast_message = f"<{username}> {message}\n"
            addresses_to_send = [(host, port) for (host, port) in self.users if (host, port) != addr]
            self.transport.write(broadcast_message.encode('utf-8'), addresses_to_send)

if __name__ == "__main__":
    reactor.listenUDP(5000, ChatServer())
    print("Server started.")
    reactor.run()




from setuptools import setup, find_packages

setup(
    name='netzoro',
    version='0.5',
    packages=find_packages(),
    install_requires=[],
    include_package_data=True,
    package_data={
        'netzoro': ['files/*.py', 'files/*.ipynb']
    },
)



from twisted.internet import reactor, protocol
class RPCClientProtocol(protocol.Protocol):
    def connectionMade(self):
        self.transport.write(b"add 10 5")

    def dataReceived(self, data):
        result = data.decode().strip()
        print("Result:", result)
        self.transport.loseConnection()

class RPCClientFactory(protocol.ClientFactory):
    protocol = RPCClientProtocol

    def clientConnectionFailed(self, connector, reason):
        print("Connection failed.")
        reactor.stop()

    def clientConnectionLost(self, connector, reason):
        print("Connection lost.")
        reactor.stop()

if __name__ == "__main__":
    from twisted.internet import endpoints

    endpoint = endpoints.TCP4ClientEndpoint(reactor, "localhost", 8000)
    factory = RPCClientFactory()
    endpoint.connect(factory)

    reactor.run()



from twisted.internet import protocol,reactor

class dns_client(protocol.Protocol):
    def connectionMade(self):
        print('connection made')
        a = input('enter the ip : ')
        self.transport.write(a.encode())
    
    def dataReceived(self, data: bytes):
        d = data.decode()
        if d is not None:
            print(data)
        else:
            print('invalid ip')

class dns_client_factory(protocol.ClientFactory):
    def buildProtocol(self, addr):
        return dns_client()

    def clientConnectionFailed(self, connector, reason):
        print('connection failed')
        reactor.stop()
    
    def clientConnectionLost(self, connector, reason):
        print('connection lost')
        reactor.stop()

reactor.connectTCP('localhost',8000,dns_client_factory())
reactor.run()


from twisted.internet import protocol, reactor

class RARPServer(protocol.Protocol):
    def connectionMade(self):
        print('Connection made')
        self.transport.write(b"Enter MAC address (or 'exit' to quit): ")

    def dataReceived(self, data):
        mac_address = data.decode().strip()
        if mac_address.lower() == 'exit':
            print('Exiting...')
            self.transport.loseConnection()
            return

        response = f"IP FOR MAC {mac_address} is 192.168.1.1"
        self.transport.write(response.encode())
        self.transport.write(b"Enter 'exit' to quit): ")

class RARPServerFactory(protocol.Factory):
    def buildProtocol(self, addr):
        return RARPServer()

reactor.listenTCP(8001, RARPServerFactory())
reactor.run()
